-- tool to ingest an NFO PATCH SHIP TABLE command and emit corresponding NML

local function embed_date(str)
  local R = ""
  for ch in str:gsub('%-', '\x00'):gmatch('.') do
    R = string.format("%s %02X", R, ch:byte())
  end
  return R.." 00"
end

local data

local function read(n)
  local n = n or 1
  local b
  b,data = data:sub(1,n),data:sub(n+1)
  return b
end

local function readi()
  return read():byte()
end

local function readstr()
  local str = ""
  while true do
    local ch = read()
    if ch == '\x00' then break end
    str = str..ch
  end
  return str
end

local function printf(...)
  io.write(string.format(...))
end

local field = {}

local function main()
  if data:match('ekranoplan00') then -- sprite table!
    local spr = {}
    for line in data:gmatch("8bpp (.-) normal chunked") do
      table.insert(spr, '  ['..(line:gsub(' +', ', '))..']')
    end
    table.insert(spr, 9, '}\n\nspriteset (spriteset_GEV_, "ekranoplan00.pcx") {')
    table.insert(spr, 1, 'spriteset (spriteset_GEV__pass, "ekranoplan00.pcx") {')
    table.insert(spr, '}\n')
    print(table.concat(spr, '\n'))
    return
  end

  local len
  local orig = data
  len,data = data:match('%d+ %* (%d+) (.*)')
  assert(len, "unsupported NFO line: '"..orig.."'")

  -- convert to binary
  data = data:gsub('%s', ''):gsub('..', function(x) return string.char(tonumber(x,16)) end)

  -- read and discard header
  assert(read(4) == '\x00\x02\x14\x01', "unsupported ACTION block")
  local idx = readi()
  printf('item(FEAT_SHIPS, item_GEV_%d) {\n  property {\n', idx)
  printf('    name: string(STR_NAME_GEV_%d);\n', idx)

  while #data > 0 do
    local cmd = readi()
    local name = 'x'..string.format('%02X', cmd)
    if field[name] then
      field[name]()
    else
      error('unsupported field 0'..name)
    end
  end

  printf('  }\n  graphics {\n')
  printf('    default: spriteset_GEV_%d;\n', idx)
  printf('    PASS: spriteset_GEV_%d_pass;\n', idx)
  printf('  }\n}\n\n')
end

function field.x00()
  printf('    introduction_date: date(%d, %d, %d);\n',
    readstr(), readstr(), readstr())
end

local function intfield(name)
  return function() printf('    %s: %d;\n', name, readi()) end
end

field.x02 = intfield 'reliability_decay';
field.x03 = intfield 'vehicle_life';
field.x04 = intfield 'model_life';

function field.x06()
  local mask = readi()
  assert(mask == 0x0F, "unsupported climate avaiability mask")
  printf('    climates_available: ALL_CLIMATES;\n');
end

function field.x08()
  local sprite = readi()
  assert(sprite == 0xFF, "unsupported sprite index")
  printf('    sprite_id: SPRITE_ID_NEW_SHIP;\n')
end

field.x09 = intfield 'is_refittable'
field.x0A = intfield 'cost_factor'

-- speed is given in halves of a km/h
-- i.e. a speed of 64 == 32 km/h
function field.x0B()
  printf('    speed: %d km/h;\n', math.floor(readi()/2))
end

-- ignore cargo data since we're using the Mars cargoes instead
function field.x0C()
  readi()
  printf('    default_cargo_type: DEFAULT_CARGO_FIRST_REFITTABLE;\n')
end

function field.x0D()
  printf('    cargo_capacity: %d;\n', readi() + readi()*256)
end

function field.x10()
  local snd = readi()
  if snd == 7 then
    printf('    sound_effect: SOUND_JET;\n')
  else
    error('unrecognized sound effect '..snd)
  end
end

function field.x11()
  read(4) -- ignore refit table in NFO
  printf('    refittable_cargo_classes: bitmask(CC_PIECE_GOODS, CC_PASSENGERS, CC_BULK, CC_COVERED, CC_REFRIGERATED);\n')
  printf('    non_refittable_cargo_classes: bitmask(CC_HAZARDOUS, CC_LIQUID);\n')
end

function field.x12()
  assert(readi() == 0, "unsupported callback flags")
end

field.x13 = intfield 'refit_cost'

function field.x14()
  printf('    ocean_speed_fraction: %f;\n', 1.0 - readi()/256)
end

function field.x15()
  printf('    canal_speed_fraction: %f;\n', 1.0 - readi()/256)
end

function field.x17()
  printf('    misc_flags: bitmask(SHIP_FLAG_2CC); // original flags: %02X\n', readi())
end

function field.x18()
  printf('    // original refittable cargo classes: 0x%04X\n', readi()+readi()*256)
end

function field.x19()
  printf('    // original non-refittable cargo classes: 0x%04X\n', readi()+readi()*256)
end

-- TODO: sprite sheets for with/without passengers
-- sprite sheets in NFO have 16 entries, first 8 are passengers, next 8 are cargo

local buf = io.read('*a')

for block in buf:gmatch('.-\n//EOF\n') do
  data = block
    :gsub('//[^\n]*', '')
    :gsub('\\w (%d+%-%d+%-%d+)', embed_date)
    :gsub('%s+', ' ')
    :gsub('^%s+', ''):gsub('%s+$','')
  main()
end

-- item(FEAT_SHIPS, item_GEV_1) {
--   property {
--     name:                 string(STR_NAME_GEV_1);
--     climates_available:   ALL_CLIMATES;
--     introduction_date:    date(1960, 01, 01);
--     vehicle_life:         5;
--     model_life:           10;
--     reliability_decay:    32;
--     cost_factor:          1;
--     running_cost_factor:  16;
--     sprite_id:            SPRITE_ID_NEW_SHIP;
--     speed:                127 km/h;
--     misc_flags:           bitmask(SHIP_FLAG_2CC);
--     refit_cost:           50;
--     is_refittable:        1;
--     default_cargo_type:   DEFAULT_CARGO_FIRST_REFITTABLE;
--     cargo_capacity:       40;
--     ocean_speed_fraction: 1.0;
--     canal_speed_fraction: 0.3;
--     visual_effect:        visual_effect(VISUAL_EFFECT_ELECTRIC, 0);
--     sound_effect:         SOUND_JET;
--     loading_speed:        255;
--     refittable_cargo_classes: bitmask(CC_PIECE_GOODS, CC_PASSENGERS, CC_BULK, CC_COVERED, CC_REFRIGERATED);
--     non_refittable_cargo_classes: bitmask(CC_HAZARDOUS, CC_LIQUID);
--   }
--   graphics {
--     default: spriteset_GEV_1;
--   }
-- }
